# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import AgGrpcServices_pb2 as AgGrpcServices__pb2


class STKGrpcServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetConnectionMetadata = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetConnectionMetadata',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ConnectionMetadata.FromString,
                )
        self.ShutDownServer = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/ShutDownServer',
                request_serializer=AgGrpcServices__pb2.ShutDownRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.GetStkApplication = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetStkApplication',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.EngineNewRoot = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EngineNewRoot',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.EngineNewRootContext = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EngineNewRootContext',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.AddRef = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/AddRef',
                request_serializer=AgGrpcServices__pb2.RefCountRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.RefCountReturn.FromString,
                )
        self.Release = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Release',
                request_serializer=AgGrpcServices__pb2.RefCountRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.RefCountReturn.FromString,
                )
        self.BatchedRelease = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/BatchedRelease',
                request_serializer=AgGrpcServices__pb2.BatchedReleaseRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.SupportsInterface = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/SupportsInterface',
                request_serializer=AgGrpcServices__pb2.SupportsInterfaceRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.SupportsInterfaceReturn.FromString,
                )
        self.Invoke = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Invoke',
                request_serializer=AgGrpcServices__pb2.InvokeRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.InvokeReturn.FromString,
                )
        self.GetProperty = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetProperty',
                request_serializer=AgGrpcServices__pb2.GetPropertyRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.GetPropertyReturn.FromString,
                )
        self.SetProperty = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/SetProperty',
                request_serializer=AgGrpcServices__pb2.SetPropertyRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.EnumerateCollection = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EnumerateCollection',
                request_serializer=AgGrpcServices__pb2.CollectionRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.CollectionReturn.FromString,
                )
        self.BatchedInvoke = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/BatchedInvoke',
                request_serializer=AgGrpcServices__pb2.BatchedInvokeRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.BatchedInvokeReturn.FromString,
                )
        self.StartEventLoop = channel.unary_stream(
                '/AgGrpcServices.STKGrpcService/StartEventLoop',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EventLoopResponse.FromString,
                )
        self.StopEventLoop = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/StopEventLoop',
                request_serializer=AgGrpcServices__pb2.EventLoopData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.AcknowledgeEvent = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/AcknowledgeEvent',
                request_serializer=AgGrpcServices__pb2.EventLoopData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.Subscribe = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Subscribe',
                request_serializer=AgGrpcServices__pb2.SubscriptionData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.Unsubscribe = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Unsubscribe',
                request_serializer=AgGrpcServices__pb2.SubscriptionData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )


class STKGrpcServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetConnectionMetadata(self, request, context):
        """Server Operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShutDownServer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStkApplication(self, request, context):
        """Object Model Entry Points
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EngineNewRoot(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EngineNewRootContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddRef(self, request, context):
        """Object/Interface Operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Release(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchedRelease(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SupportsInterface(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Invoke(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EnumerateCollection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchedInvoke(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartEventLoop(self, request, context):
        """Event Handling
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopEventLoop(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AcknowledgeEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Subscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Unsubscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_STKGrpcServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetConnectionMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetConnectionMetadata,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ConnectionMetadata.SerializeToString,
            ),
            'ShutDownServer': grpc.unary_unary_rpc_method_handler(
                    servicer.ShutDownServer,
                    request_deserializer=AgGrpcServices__pb2.ShutDownRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'GetStkApplication': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStkApplication,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'EngineNewRoot': grpc.unary_unary_rpc_method_handler(
                    servicer.EngineNewRoot,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'EngineNewRootContext': grpc.unary_unary_rpc_method_handler(
                    servicer.EngineNewRootContext,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'AddRef': grpc.unary_unary_rpc_method_handler(
                    servicer.AddRef,
                    request_deserializer=AgGrpcServices__pb2.RefCountRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.RefCountReturn.SerializeToString,
            ),
            'Release': grpc.unary_unary_rpc_method_handler(
                    servicer.Release,
                    request_deserializer=AgGrpcServices__pb2.RefCountRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.RefCountReturn.SerializeToString,
            ),
            'BatchedRelease': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchedRelease,
                    request_deserializer=AgGrpcServices__pb2.BatchedReleaseRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'SupportsInterface': grpc.unary_unary_rpc_method_handler(
                    servicer.SupportsInterface,
                    request_deserializer=AgGrpcServices__pb2.SupportsInterfaceRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.SupportsInterfaceReturn.SerializeToString,
            ),
            'Invoke': grpc.unary_unary_rpc_method_handler(
                    servicer.Invoke,
                    request_deserializer=AgGrpcServices__pb2.InvokeRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.InvokeReturn.SerializeToString,
            ),
            'GetProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProperty,
                    request_deserializer=AgGrpcServices__pb2.GetPropertyRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.GetPropertyReturn.SerializeToString,
            ),
            'SetProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.SetProperty,
                    request_deserializer=AgGrpcServices__pb2.SetPropertyRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'EnumerateCollection': grpc.unary_unary_rpc_method_handler(
                    servicer.EnumerateCollection,
                    request_deserializer=AgGrpcServices__pb2.CollectionRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.CollectionReturn.SerializeToString,
            ),
            'BatchedInvoke': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchedInvoke,
                    request_deserializer=AgGrpcServices__pb2.BatchedInvokeRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.BatchedInvokeReturn.SerializeToString,
            ),
            'StartEventLoop': grpc.unary_stream_rpc_method_handler(
                    servicer.StartEventLoop,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.EventLoopResponse.SerializeToString,
            ),
            'StopEventLoop': grpc.unary_unary_rpc_method_handler(
                    servicer.StopEventLoop,
                    request_deserializer=AgGrpcServices__pb2.EventLoopData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'AcknowledgeEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.AcknowledgeEvent,
                    request_deserializer=AgGrpcServices__pb2.EventLoopData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'Subscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.Subscribe,
                    request_deserializer=AgGrpcServices__pb2.SubscriptionData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'Unsubscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.Unsubscribe,
                    request_deserializer=AgGrpcServices__pb2.SubscriptionData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'AgGrpcServices.STKGrpcService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class STKGrpcService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetConnectionMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetConnectionMetadata',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ConnectionMetadata.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ShutDownServer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/ShutDownServer',
            AgGrpcServices__pb2.ShutDownRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStkApplication(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetStkApplication',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EngineNewRoot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EngineNewRoot',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EngineNewRootContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EngineNewRootContext',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddRef(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/AddRef',
            AgGrpcServices__pb2.RefCountRequest.SerializeToString,
            AgGrpcServices__pb2.RefCountReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Release(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Release',
            AgGrpcServices__pb2.RefCountRequest.SerializeToString,
            AgGrpcServices__pb2.RefCountReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchedRelease(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/BatchedRelease',
            AgGrpcServices__pb2.BatchedReleaseRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SupportsInterface(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/SupportsInterface',
            AgGrpcServices__pb2.SupportsInterfaceRequest.SerializeToString,
            AgGrpcServices__pb2.SupportsInterfaceReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Invoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Invoke',
            AgGrpcServices__pb2.InvokeRequest.SerializeToString,
            AgGrpcServices__pb2.InvokeReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetProperty',
            AgGrpcServices__pb2.GetPropertyRequest.SerializeToString,
            AgGrpcServices__pb2.GetPropertyReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/SetProperty',
            AgGrpcServices__pb2.SetPropertyRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EnumerateCollection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EnumerateCollection',
            AgGrpcServices__pb2.CollectionRequest.SerializeToString,
            AgGrpcServices__pb2.CollectionReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchedInvoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/BatchedInvoke',
            AgGrpcServices__pb2.BatchedInvokeRequest.SerializeToString,
            AgGrpcServices__pb2.BatchedInvokeReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartEventLoop(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/AgGrpcServices.STKGrpcService/StartEventLoop',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.EventLoopResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopEventLoop(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/StopEventLoop',
            AgGrpcServices__pb2.EventLoopData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AcknowledgeEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/AcknowledgeEvent',
            AgGrpcServices__pb2.EventLoopData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Subscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Subscribe',
            AgGrpcServices__pb2.SubscriptionData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Unsubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Unsubscribe',
            AgGrpcServices__pb2.SubscriptionData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
