# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: AgGrpcServices.proto
# Protobuf Python Version: 5.26.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x41gGrpcServices.proto\x12\x0e\x41gGrpcServices\"+\n\x0bInterfaceID\x12\r\n\x05\x64\x61ta1\x18\x01 \x01(\x06\x12\r\n\x05\x64\x61ta2\x18\x02 \x01(\x06\"E\n\tSTKObject\x12\r\n\x05value\x18\x01 \x01(\x06\x12)\n\x04guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\"\x0e\n\x0c\x45mptyMessage\"\x0b\n\tNullValue\"]\n\x12\x43onnectionMetadata\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0f\n\x07release\x18\x02 \x01(\r\x12\x0e\n\x06update\x18\x03 \x01(\r\x12\x15\n\rconnection_id\x18\x04 \x01(\r\"a\n\x0fShutDownRequest\x12\x31\n\x0e\x61pp_to_release\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x1b\n\x13shutdown_stkruntime\x18\x02 \x01(\x08\"3\n\tObjReturn\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\"9\n\x0fRefCountRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\"\x1f\n\x0eRefCountReturn\x12\r\n\x05\x63ount\x18\x01 \x01(\r\"C\n\x15\x42\x61tchedReleaseRequest\x12*\n\x07objects\x18\x01 \x03(\x0b\x32\x19.AgGrpcServices.STKObject\"w\n\x18SupportsInterfaceRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\")\n\x17SupportsInterfaceReturn\x12\x0e\n\x06result\x18\x01 \x01(\x08\"\x93\x03\n\x07Variant\x12\x34\n\x06values\x18\x01 \x03(\x0b\x32$.AgGrpcServices.Variant.VariantValue\x12&\n\x1enum_columns_in_repeated_values\x18\x02 \x01(\r\x1a\xa9\x02\n\x0cVariantValue\x12(\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x11\n\x07str_val\x18\x02 \x01(\tH\x00\x12\x18\n\x0esigned_int_val\x18\x03 \x01(\x10H\x00\x12\x1a\n\x10unsigned_int_val\x18\x04 \x01(\x06H\x00\x12\x14\n\ndouble_val\x18\x05 \x01(\x01H\x00\x12\x12\n\x08\x62ool_val\x18\x06 \x01(\x08H\x00\x12)\n\x04null\x18\x07 \x01(\x0b\x32\x19.AgGrpcServices.NullValueH\x00\x12/\n\x0cnested_array\x18\x08 \x01(\x0b\x32\x17.AgGrpcServices.VariantH\x00\x12\x17\n\robj_future_id\x18\t \x01(\rH\x00\x42\x07\n\x05Value\"\x80\x01\n\x12GetPropertyRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x03 \x01(\r\"=\n\x11GetPropertyReturn\x12(\n\x07variant\x18\x01 \x01(\x0b\x32\x17.AgGrpcServices.Variant\"\xaa\x01\n\x12SetPropertyRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x03 \x01(\r\x12(\n\x07variant\x18\x04 \x01(\x0b\x32\x17.AgGrpcServices.Variant\"\xd8\x01\n\x10STKObjectPromise\x12\n\n\x02id\x18\x01 \x01(\r\x12(\n\x03obj\x18\x02 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x17\n\robj_future_id\x18\x03 \x01(\rH\x00\x12\x33\n\x0einterface_guid\x18\x04 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x05 \x01(\r\x12%\n\x04\x61rgs\x18\x06 \x03(\x0b\x32\x17.AgGrpcServices.VariantB\n\n\x08instance\"\xe4\x01\n\rInvokeRequest\x12(\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x32\n\x06\x66uture\x18\x02 \x01(\x0b\x32 .AgGrpcServices.STKObjectPromiseH\x00\x12\x33\n\x0einterface_guid\x18\x03 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x04 \x01(\r\x12%\n\x04\x61rgs\x18\x05 \x03(\x0b\x32\x17.AgGrpcServices.VariantB\n\n\x08instance\"<\n\x0cInvokeReturn\x12,\n\x0breturn_vals\x18\x01 \x03(\x0b\x32\x17.AgGrpcServices.Variant\"e\n\x14\x42\x61tchedInvokeRequest\x12/\n\x08requests\x18\x01 \x03(\x0b\x32\x1d.AgGrpcServices.InvokeRequest\x12\x1c\n\x14\x66uture_ids_to_return\x18\x02 \x03(\r\"\xab\x01\n\x13\x42\x61tchedInvokeReturn\x12\x46\n\rbound_futures\x18\x01 \x03(\x0b\x32/.AgGrpcServices.BatchedInvokeReturn.BoundFuture\x1aL\n\x0b\x42oundFuture\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x15\n\robj_future_id\x18\x02 \x01(\r\"N\n\x11\x43ollectionRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x11\n\tmax_items\x18\x02 \x01(\r\"S\n\x10\x43ollectionReturn\x12&\n\x05items\x18\x01 \x03(\x0b\x32\x17.AgGrpcServices.Variant\x12\x17\n\x0f\x64one_collecting\x18\x02 \x01(\x08\"\x9a\x01\n\x10SubscriptionData\x12\x15\n\revent_loop_id\x18\x01 \x01(\r\x12&\n\x03obj\x18\x02 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\revent_handler\x18\x03 \x01(\x0e\x32\x1c.AgGrpcServices.EventHandler\x12\x12\n\nevent_name\x18\x04 \x01(\t\"8\n\rEventLoopData\x12\x15\n\revent_loop_id\x18\x01 \x01(\r\x12\x10\n\x08\x65vent_id\x18\x02 \x01(\r\"\x8d\x01\n\x11\x45ventLoopResponse\x12\x36\n\x0csubscription\x18\x01 \x01(\x0b\x32 .AgGrpcServices.SubscriptionData\x12.\n\rcallback_args\x18\x02 \x03(\x0b\x32\x17.AgGrpcServices.Variant\x12\x10\n\x08\x65vent_id\x18\x03 \x01(\r*\x83\x02\n\x0c\x45ventHandler\x12\x1b\n\x17\x65IAgStkObjectRootEvents\x10\x00\x12\x1d\n\x19\x65IAgSTKXApplicationEvents\x10\x01\x12\x1e\n\x1a\x65IAgStkGraphicsSceneEvents\x10\x02\x12$\n eIAgStkGraphicsKmlGraphicsEvents\x10\x03\x12(\n$eIAgStkGraphicsImageCollectionEvents\x10\x04\x12*\n&eIAgStkGraphicsTerrainCollectionEvents\x10\x05\x12\x1b\n\x17\x65IAgUiApplicationEvents\x10\x06\x32\xd7\x0c\n\x0eSTKGrpcService\x12[\n\x15GetConnectionMetadata\x12\x1c.AgGrpcServices.EmptyMessage\x1a\".AgGrpcServices.ConnectionMetadata\"\x00\x12Q\n\x0eShutDownServer\x12\x1f.AgGrpcServices.ShutDownRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12N\n\x11GetStkApplication\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12J\n\rEngineNewRoot\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12Q\n\x14\x45ngineNewRootContext\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12K\n\x06\x41\x64\x64Ref\x12\x1f.AgGrpcServices.RefCountRequest\x1a\x1e.AgGrpcServices.RefCountReturn\"\x00\x12L\n\x07Release\x12\x1f.AgGrpcServices.RefCountRequest\x1a\x1e.AgGrpcServices.RefCountReturn\"\x00\x12W\n\x0e\x42\x61tchedRelease\x12%.AgGrpcServices.BatchedReleaseRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12h\n\x11SupportsInterface\x12(.AgGrpcServices.SupportsInterfaceRequest\x1a\'.AgGrpcServices.SupportsInterfaceReturn\"\x00\x12G\n\x06Invoke\x12\x1d.AgGrpcServices.InvokeRequest\x1a\x1c.AgGrpcServices.InvokeReturn\"\x00\x12V\n\x0bGetProperty\x12\".AgGrpcServices.GetPropertyRequest\x1a!.AgGrpcServices.GetPropertyReturn\"\x00\x12Q\n\x0bSetProperty\x12\".AgGrpcServices.SetPropertyRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12\\\n\x13\x45numerateCollection\x12!.AgGrpcServices.CollectionRequest\x1a .AgGrpcServices.CollectionReturn\"\x00\x12\\\n\rBatchedInvoke\x12$.AgGrpcServices.BatchedInvokeRequest\x1a#.AgGrpcServices.BatchedInvokeReturn\"\x00\x12U\n\x0eStartEventLoop\x12\x1c.AgGrpcServices.EmptyMessage\x1a!.AgGrpcServices.EventLoopResponse\"\x00\x30\x01\x12N\n\rStopEventLoop\x12\x1d.AgGrpcServices.EventLoopData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12Q\n\x10\x41\x63knowledgeEvent\x12\x1d.AgGrpcServices.EventLoopData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12M\n\tSubscribe\x12 .AgGrpcServices.SubscriptionData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12O\n\x0bUnsubscribe\x12 .AgGrpcServices.SubscriptionData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'AgGrpcServices_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENTHANDLER']._serialized_start=2844
  _globals['_EVENTHANDLER']._serialized_end=3103
  _globals['_INTERFACEID']._serialized_start=40
  _globals['_INTERFACEID']._serialized_end=83
  _globals['_STKOBJECT']._serialized_start=85
  _globals['_STKOBJECT']._serialized_end=154
  _globals['_EMPTYMESSAGE']._serialized_start=156
  _globals['_EMPTYMESSAGE']._serialized_end=170
  _globals['_NULLVALUE']._serialized_start=172
  _globals['_NULLVALUE']._serialized_end=183
  _globals['_CONNECTIONMETADATA']._serialized_start=185
  _globals['_CONNECTIONMETADATA']._serialized_end=278
  _globals['_SHUTDOWNREQUEST']._serialized_start=280
  _globals['_SHUTDOWNREQUEST']._serialized_end=377
  _globals['_OBJRETURN']._serialized_start=379
  _globals['_OBJRETURN']._serialized_end=430
  _globals['_REFCOUNTREQUEST']._serialized_start=432
  _globals['_REFCOUNTREQUEST']._serialized_end=489
  _globals['_REFCOUNTRETURN']._serialized_start=491
  _globals['_REFCOUNTRETURN']._serialized_end=522
  _globals['_BATCHEDRELEASEREQUEST']._serialized_start=524
  _globals['_BATCHEDRELEASEREQUEST']._serialized_end=591
  _globals['_SUPPORTSINTERFACEREQUEST']._serialized_start=593
  _globals['_SUPPORTSINTERFACEREQUEST']._serialized_end=712
  _globals['_SUPPORTSINTERFACERETURN']._serialized_start=714
  _globals['_SUPPORTSINTERFACERETURN']._serialized_end=755
  _globals['_VARIANT']._serialized_start=758
  _globals['_VARIANT']._serialized_end=1161
  _globals['_VARIANT_VARIANTVALUE']._serialized_start=864
  _globals['_VARIANT_VARIANTVALUE']._serialized_end=1161
  _globals['_GETPROPERTYREQUEST']._serialized_start=1164
  _globals['_GETPROPERTYREQUEST']._serialized_end=1292
  _globals['_GETPROPERTYRETURN']._serialized_start=1294
  _globals['_GETPROPERTYRETURN']._serialized_end=1355
  _globals['_SETPROPERTYREQUEST']._serialized_start=1358
  _globals['_SETPROPERTYREQUEST']._serialized_end=1528
  _globals['_STKOBJECTPROMISE']._serialized_start=1531
  _globals['_STKOBJECTPROMISE']._serialized_end=1747
  _globals['_INVOKEREQUEST']._serialized_start=1750
  _globals['_INVOKEREQUEST']._serialized_end=1978
  _globals['_INVOKERETURN']._serialized_start=1980
  _globals['_INVOKERETURN']._serialized_end=2040
  _globals['_BATCHEDINVOKEREQUEST']._serialized_start=2042
  _globals['_BATCHEDINVOKEREQUEST']._serialized_end=2143
  _globals['_BATCHEDINVOKERETURN']._serialized_start=2146
  _globals['_BATCHEDINVOKERETURN']._serialized_end=2317
  _globals['_BATCHEDINVOKERETURN_BOUNDFUTURE']._serialized_start=2241
  _globals['_BATCHEDINVOKERETURN_BOUNDFUTURE']._serialized_end=2317
  _globals['_COLLECTIONREQUEST']._serialized_start=2319
  _globals['_COLLECTIONREQUEST']._serialized_end=2397
  _globals['_COLLECTIONRETURN']._serialized_start=2399
  _globals['_COLLECTIONRETURN']._serialized_end=2482
  _globals['_SUBSCRIPTIONDATA']._serialized_start=2485
  _globals['_SUBSCRIPTIONDATA']._serialized_end=2639
  _globals['_EVENTLOOPDATA']._serialized_start=2641
  _globals['_EVENTLOOPDATA']._serialized_end=2697
  _globals['_EVENTLOOPRESPONSE']._serialized_start=2700
  _globals['_EVENTLOOPRESPONSE']._serialized_end=2841
  _globals['_STKGRPCSERVICE']._serialized_start=3106
  _globals['_STKGRPCSERVICE']._serialized_end=4729
# @@protoc_insertion_point(module_scope)
